// Copyright (C) 2001 Fabio Lopiano <flopiano@bigfoot.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef NET_ARRAY_H
#define NET_ARRAY_H

#include "netobject.h"

// T is the type of the object contained
// N is the size of the array 
template <class T, int N>
class NetArray : public NetObject
{
    T data[N];
protected:
    virtual void netWrite (NetBuffer &nb) const 
      {  for(int i=0; i<N; i++) nb << data[i];  }
    
    virtual void netRead  (NetBuffer &nb) 
      { for(int i=0; i<N; i++) nb >> data[i]; }

public:

    NetArray () 
    { 
        netClear(); 
    }

    NetArray (const NetObject     &anObject)
    { 
        netCopy(anObject);   
    }

    NetArray (const NetBuffer     &aBuffer )
    { 
        netCopy(aBuffer);    
    }
    
    inline NetArray & operator =(const NetObject &anObject) 
    { 
        netCopy(anObject); return *this; 
    }

    inline NetArray & operator =(const NetBuffer &aBuffer)
    { 
        netCopy(aBuffer);return *this; 
    }

    operator       T*()       { return data; }
    operator const T*() const { return data; }
};

#endif

