//
// Copyright (C) 2001 Fabio Lopiano <flopiano@bigfoot.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#ifndef NETBUFFER_H
#define NETBUFFER_H

#include "buffer.h"

#if defined (WIN32)
	#include <winsock2.h>	// ntoh*
#else
	#include <arpa/inet.h>  // ntoh*
#endif

class NetBuffer : public Buffer
{
    size_t theReadPosition;
    size_t theWritePosition;

    void reset() 
    {
        theReadPosition  = 0;
        theWritePosition = 0;
    }
    
public:
    explicit NetBuffer(size_t s=0);    
    NetBuffer(const Buffer &aBuffer);
    NetBuffer(const char *aCharBuffer,size_t aSize);
    ~NetBuffer() {}

    inline size_t bytesRead() const
    {
         return theReadPosition; 
    }

    inline size_t bytesWritten() const 
    {
        return theWritePosition; 
    }
    
    bool get(char *aBuffer, size_t aSize);
    void put(const char *aBuffer, size_t aSize);

};

inline bool operator==(const NetBuffer & a, const NetBuffer &b)
{
    if(a.size() != b.size()) return false;
    
    return memcmp(a, b, a.size() )==0;
}

inline bool operator!=(const NetBuffer & a, const NetBuffer &b)
{    
    return !( a == b );
}

// --- operators for chars ---

inline NetBuffer & operator << (NetBuffer &n, char c) 
{
    n.put(&c , 1); 
    return n; 
}

inline NetBuffer & operator >> (NetBuffer &n, char &c)
{
    if ( !n.get (&c , 1)) 
        c = 0;

    return n;
}

inline NetBuffer & operator << (NetBuffer &n, unsigned char c)
{
    n.put(reinterpret_cast<char *>(&c) , 1); 
    return n; 

}

inline NetBuffer & operator >> (NetBuffer &n, unsigned char &c)
{
    if ( !n.get (reinterpret_cast<char *>(&c) , 1)) 
        c = 0;

    return n;
}

// --- operators for shorts ---

inline NetBuffer & operator << (NetBuffer &n, short s)
{
    s = htons(s);
    n.put(reinterpret_cast<char *>(&s) , 2);
    return n;
}

inline NetBuffer & operator >> (NetBuffer &n, short &s)
{
    short val =
        n.get(reinterpret_cast<char *>(&s) , 2)
        ? htons(s) : 0;

    return n;    
}

inline NetBuffer & operator << (NetBuffer &n, unsigned short s)
{
    s = htons(s);
    n.put(reinterpret_cast<char *>(&s) , 2);
    return n;
}
inline NetBuffer & operator >> (NetBuffer &n, unsigned short &s)
{
    unsigned short val =
        n.get(reinterpret_cast<char *>(&s) , 2)
        ? htons(s) : 0;

    return n;    
}

// --- operators for ints ---

inline NetBuffer & operator << (NetBuffer &n, int i)
{
    i = htonl(i);
    n.put(reinterpret_cast<char *>(&i) , 4);
    return n;
}

inline NetBuffer & operator >> (NetBuffer &n, int &i)
{
    int val =
        n.get(reinterpret_cast<char *>(&i) , 4)
        ? htonl(i) : 0;
    return n;    
}

inline NetBuffer & operator << (NetBuffer &n, unsigned int i)
{
    i = htonl(i);
    n.put(reinterpret_cast<char *>(&i) , 4);
    return n;
}

inline NetBuffer & operator >> (NetBuffer &n, unsigned int &i)
{
    unsigned int val =
        n.get(reinterpret_cast<char *>(&i) , 4)
        ? htonl(i) : 0;
    return n;    
}

// --- operators for Buffer

inline NetBuffer & operator << (NetBuffer &n, const Buffer &b)
{
    n.put( b , b.size() );

    return n;
}                                    

inline NetBuffer & operator >> (NetBuffer &n, Buffer &b)
{
    n.get( b.buffer() , b.size() );

    return n;
}

#endif

