//
// Copyright (C) 2001 Fabio Lopiano <flopiano@bigfoot.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#ifndef NET_OBJECT_H
#define NET_OBJECT_H

#include "netbuffer.h"

class NetObject 
{
protected:

    virtual void netWrite (NetBuffer &nb) const = 0;
    virtual void netRead  (NetBuffer &nb) = 0;
   
    void netCopy (const NetObject &nb);
    void netCopy (const Buffer &nb);

public:

    NetObject () {}
    virtual ~NetObject() {}

    void netClear() ;

    unsigned int netSize() const;

    bool operator==(const NetObject &nb) const;
    bool operator!=(const NetObject &nb) const { return *this != nb; }    

    Buffer toBuffer() const
    {
	   NetBuffer nbuf( netSize() );
	   netWrite (nbuf);
 	   return nbuf;
    }

    friend inline NetBuffer & 
    operator >> (NetBuffer &nbuf, NetObject &obj)
    {
	   obj.netRead( nbuf );
	   return nbuf;
    }

    friend inline NetBuffer & 
    operator << (NetBuffer &nbuf, const NetObject &obj)
    {
	   obj.netWrite( nbuf );
	   return nbuf;
    }

};

//
// T must inherit from NetObject
//
template <class T>
class NetObjectT : public T
{
public:

    NetObjectT () 
    { 
        netClear(); 
    }
    NetObjectT (const NetObject &anObject)
    { 
        netCopy(anObject);   
    }
    NetObjectT (const Buffer &aBuffer )
    { 
        netCopy(aBuffer);    
    }
    
    inline NetObjectT & operator =(const NetObject &anObject) 
    { 
        netCopy(anObject); 
        return *this; 
    }

    inline NetObjectT & operator =(const Buffer &aBuffer)
    { 
        netCopy(aBuffer);
        return *this; 
    }
};

#endif
