// Copyright (C) 2001 Fabio Lopiano <flopiano@bigfoot.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#ifndef NET_VECTOR_H
#define NET_VECTOR_H


#include <vector>
#include "netobject.h"


// T is the type of the objects contained
// C is the type of the counter
template <class T, typename C=unsigned char>
class NetVector : 
    public NetObject, 
    public std::vector<T>
{

protected:

    virtual void netWrite (NetBuffer &nb) const 
    {        
        C counter = size();

        nb << counter;

	for( const_iterator it = begin();
	     it != end() ; ++it )
		nb << (*it);
    }
    
    virtual void netRead  (NetBuffer &nb) 
    { 
        erase(begin(),end());

        C counter;
        T item;
        nb >> counter;
        for(int i=0; i<counter; i++) 
        {
            nb >> item ;
            push_back(item);
        }
    }
public:

    explicit NetVector (size_t n=0) : std::vector<T>(n) { netClear(); }

    NetVector (const NetObject &anObject)
    { 
        netCopy(anObject);   
    }
    
    NetVector (const NetBuffer &aBuffer )
    { 
        netCopy(aBuffer);    
    }
        
    inline NetVector & operator =(const NetObject &anObject) 
    { 
        netCopy(anObject); 
        return *this; 
    }

    inline NetVector & operator =(const NetBuffer &aBuffer)
    {
         netCopy(aBuffer);
         return *this; 
    }
};

#endif

